var ws, LastNav = null, Timer = null;

$(document).ready(function() {
	if ("WebSocket" in window) {
		var Host = window.location.hostname;
		ws = new WebSocket("ws://"+Host+":8214", "Lux_WS");
		ws.onopen = function() {
			password_prompt(function(PW) {
				if(!PW.length) PW = '0';
				ws.send("LOGIN;"+PW);
				Timer = setInterval(requestPageRefresh, 1000);
			});
		};
		
		ws.onmessage = function (evt) {
			processResponse($.parseXML(evt.data));
	   };

		ws.onclose = function(evt) {
			alert("Lost connection to controller!");
			if(Timer) clearInterval(Timer);
			ws = null;
		}
	} else alert("WebSocket NOT supported by your Browser!");
});

function requestPageRefresh()
{
	if(ws) ws.send("REFRESH");
}

function BuildChildren(element, Container, ItemFunc) {
	var El			 = $(element),
		Children	 = El.children('item'),								//Fetches only the direct children of the node.
		BaseID		 = El.attr('id'),
		Item		 = { 
			"ID":			BaseID,
			"options":		new Array(),
			"Depth":		0,
			"hasChildren":	Children.length > 0,
			"Container":	Container,
			"ChildFunc":	ItemFunc
		}, Diver	 = Children,
		Values		 = new Array();
	
	while(Diver.length > 0) {
		Diver		 = Diver.children("item");
		Item.Depth	+= 1;
	}

	El.children(':not(item)').each(function() {
		var Value	 = $(this).text();
		if(this.nodeName == "option")		Item["options"].push({ Text: Value, Val: $(this).attr('value') });
		else if(this.nodeName == "value")	Values.push(Value);
		else								Item[this.nodeName]	= Value;
	});

	if(Values.length > 1) {
		var BaseName	= Item.name,
			Index		= 0;
		while(true) {
			Item.value	= Values[Index];
			Item.name	= BaseName + " " + ++Index;
			if(Index == Values.length) break;
			Container.append(ItemFunc(Item));
			Item.ID		= BaseID + "_" + Index;
		}
	} else if(Values.length > 0) Item.value = Values[0];

	Container.append(ItemFunc(Item));
	Children.each(function() {
		BuildChildren(this, Item.Container, Item.ChildFunc);
	});
}

//------------------------------------------------------------------------
function processResponse(xmlDoc) {
	var XML			= $(xmlDoc);
	var OpNode		= XML.find(':root'), 
		Cont_ID		= OpNode.prop('tagName');

	if(Cont_ID.toUpperCase() == "VALUES") {
		XML.find("item").each(function() {
			var field	= $(this),
				BaseID	= field.attr('id'),
				Values	= field.children('value');
			refreshMultipleLines(BaseID, Values);
		});
	} else {
		var Container = $('#'+Cont_ID), ItemFunc;

		Container.empty();
		$('#SubmitBox').remove();

		if(Cont_ID == "Navigation") {
			ItemFunc = function(Item) {
				var Line = null;
				if(hasNotEmptyProperty(Item, 'name'))
					Line = buildNavLine(Item);
				if(Item.hasChildren) {
					Item.Container.append(Line);
					Item.Container = Line = $("<ul>");
				}		
				return Line;
			}
		} else {
			var hasSettings	= (XML.find('raw').length > 0);
			ItemFunc = function(Item) {
				var Line;
				Item.hasSettings		= hasSettings;
				if(Item.hasChildren) {
					if(Item.Depth == 1) {
						Line			= $("<fieldset>");
						Line.append("<legend>"+Item.name+"</legend>");
						Item.Container	= $("<table>").appendTo(Line);
					} else Line = Item.Container = $("<div>");
				} else Line = buildLine(Item);
				return Line;
			}
			
			if(hasSettings) buildSubmitBox();
		}
		BuildChildren(OpNode, Container, ItemFunc, 0);
		$("#EVO_overlay_container").hide();
	}
}

function refreshMultipleLines(BaseID, Values) {
	var ID	= BaseID,
		i	= 0;
	while(i < Values.length) {
		var Value = Values[i];
		refreshLine(
			ID, 
			document.getElementById(ID), 
			Value.innerHTML || Value.textContent
		);
		ID	= BaseID + "_" + ++i;
	}
}

function refreshLine(ID, Item, Value) {
	if(Item && Value.length > 0) {
		var Set = document.getElementById('set_'+ID);
		Item.innerHTML = Value;
		if(Set && Set.nodeName == "TD")
			Set.innerHTML = Value;
	}
}

function buildLine(Item) {
	var NameCol		= $('<td class="output_field_long">'+Item.name+'</td>'),
		Line		= $('<tr>').append(NameCol);
	if(hasNotEmptyProperty(Item, "value")) buildValueLine(Line, Item);
	else NameCol.attr('colspan', '3');
	return Line;
}

function hasNotEmptyProperty(Item, PropertyName) {
	return (Item.hasOwnProperty(PropertyName) && Item[PropertyName].length > 0);
}

function buildNavLine(Item) {
	var Line = $('<li>'+Item.name+'</li>');
	Line.attr('class', 'NavLink');
	Line.click(function() {
		var This = $(this);
		if(This != LastNav) {
			$("#EVO_overlay_container").show();
			This.removeAttr("class");
			if(LastNav != null) LastNav.addClass("NavLink");
			LastNav = This;
			ws.send("GET;"+Item.ID);
		}
		return false;
	});
	return Line;
}

function buildValueLine(Line, Item) {
	var ValCol = $('<td id='+Item.ID+' class="output_field">'+Item.value+'</td>'),
		NewEl = null;
	Line.append(ValCol);
	if(Item.hasOwnProperty("raw"))				NewEl = buildSettingLine(Line, Item);
	else if(Item.hasOwnProperty("multiple"))	NewEl = buildTooltip(ValCol, Item);

	if(NewEl) {
		var SettingColumn = $('<td class="output_field">');
		NewEl.val(Item.raw);
		NewEl.change(function() {
			if(ws) {
				var El = $(this);
				ws.send("SET;"+El.attr('id')+";"+El.val()*El.data('Div'));
			}
		});
		Line.append(SettingColumn.append(NewEl));
		if(Item.hasOwnProperty("min")) {
			NewEl.wrap('<span class="input_span" />');
			if(Item.unit.length > 0)
				SettingColumn.prepend('<span class="Unit">'+Item.unit+'</span>');
		}
	} else if(Item.hasSettings) Line.append(ValCol.clone().attr('id', 'set_'+Item.ID));
	else ValCol.attr('colspan', '2');
}

function buildSettingLine(Line, Item) {
	var NewEl = null;
	if(Item.options.length) {
		if(Item.hasOwnProperty("multiple")) NewEl = buildCheckListSetting(Item);
		else								NewEl = buildComboSetting(Item);
		NewEl.data('Div', 1);
	} else if(Item.hasOwnProperty("min"))	NewEl = buildTemperatureSetting(Line, Item);
	return NewEl;
}

function buildCheckListSetting(Item) {
	var Container	= $('<div class="CheckList">'),
		NewEl		= $('<input type="hidden" id=set_'+Item.ID+'">'),
		Options		= Item.options;
	for(var i = 0; i < Options.length; ++i) {
		var Cbx = $('<input type="checkbox" value="'+Options[i].Val+'">'+Options[i].Text+'</input>');
		Cbx.prop('checked', (Item.raw&(1 << i)) > 0);
		Cbx.change(function() {
			var Val = NewEl.val(), Bit = (1 << Cbx.val());
			if(Cbx.prop('checked')) Val |= Bit;
			else					Val &= ~Bit;
			NewEl.val(Val);
		});
		Container.append(Cbx);
	}
	return NewEl;
}

function buildComboSetting(Item) {
	var NewEl	= $('<select id=set_'+Item.ID+'>'),
		Options = Item.options;

	for(var i = 0; i < Options.length; ++i)
		NewEl.append('<option value="'+Options[i].Val+'">'+Options[i].Text+'</option>');
	return NewEl;
}

function buildTemperatureSetting(Line, Item) {
	var NewEl = $('<input type="number" min="'+(Item.min/Item.div)+'" max="'+(Item.max/Item.div)+'" step="'+(Item.step/Item.div)+'" id="set_'+Item.ID+'">');
	Item.raw /= Item.div;
	NewEl.data('Div', Item.div);
	return NewEl;
}

function buildTooltip(Container, Item) {
	var ToolTip = $('<span class="tooltip">'),
		Options	= Item.options,
		value	= Item.value;
	for(var i = 0; i < Options.length; ++i)
		if(value&(1 << i)) ToolTip.append(Options[i].Text+"<br />");
	Container.addClass("toolbox")
	Container.append(ToolTip);
}

function buildSubmitBox() {
	var Submit = $('<input type="submit">'), Cancel = $('<input type="reset">'), Line = $("<p id='SubmitBox'>");
	Submit.click(function() {
		if(ws) ws.send("SAVE;1");
		return false;
	});
	
	Cancel.click(function() {
		if(ws) ws.send("SAVE;0");
		return false;
	});
	Line.append(Submit);
	Line.append(" ");
	Line.append(Cancel);
	$('#Content').after(Line);
}
//------------------------------------------------------------------------

function password_prompt(callback) {
	var input = $("#password_prompt_input"), 
		prompt = document.getElementById('password_prompt'),
		button = $('#password_submit_button');

	var submit = function() {
		callback(input.val());
		prompt.removeAttribute('style');
		input.val('');
	};
	
	input.on('keydown', function(e) { 
			if (e.keyCode == 13) {
				$(this).off();
				submit(); 
			}
		}
	);
	button.one('click', submit);
	
	prompt.style.display = 'block';
	input.focus();
}
